@@ -52,7 +52,7 @@ kbusControlGetCaps
     NvU32 caps = 0;
 
     // if the Chip is integrated.
-    if (IsTEGRA(pGpu) || pGpu->getProperty(pGpu, PDB_PROP_GPU_ZERO_FB))
+    if (IsTEGRA(pGpu) || pGpu->pGpuArch->bGpuArchIsZeroFb)
     {
         caps |= NV2080_CTRL_BUS_INFO_CAPS_CHIP_INTEGRATED;
     }
@@ -83,50 +83,6 @@ _kbusGetHostCaps(OBJGPU *pGpu, NvU8 *pHostCaps)
     return status;
 }
 
-static NV_STATUS
-_getAspmL1FlagsSendRpc
-(
-    OBJGPU *pGpu,
-    NvBool *bCyaMaskL1,
-    NvBool *bEnableAspmDtL1
-)
-{
-    RM_API    *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
-    NV_STATUS  rmStatus;
-    NV2080_CTRL_INTERNAL_BIF_GET_ASPM_L1_FLAGS_PARAMS *pBifAspmL1Flags;
-
-    // Allocate memory for the command parameter
-    pBifAspmL1Flags = portMemAllocNonPaged(sizeof(*pBifAspmL1Flags));
-    if (pBifAspmL1Flags == NULL)
-    {
-        NV_PRINTF(LEVEL_ERROR, "Could not allocate pBifAspmL1Flags.");
-        rmStatus = NV_ERR_NO_MEMORY;
-        goto _getAspmL1FlagsSendRpc_exit;
-    }
-    portMemSet(pBifAspmL1Flags, 0, sizeof(*pBifAspmL1Flags));
-
-    // Send RPC to GSP to get physical BIF PDBs
-    rmStatus = pRmApi->Control(pRmApi,
-                                pGpu->hInternalClient,
-                                pGpu->hInternalSubdevice,
-                                NV2080_CTRL_CMD_INTERNAL_BIF_GET_ASPM_L1_FLAGS,
-                                pBifAspmL1Flags, sizeof(*pBifAspmL1Flags));
-
-    if (NV_OK != rmStatus)
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                    "Error 0x%x receiving bus ASPM disable flags from GSP.\n", rmStatus);
-        goto _getAspmL1FlagsSendRpc_exit;
-    }
-
-    *bCyaMaskL1      = pBifAspmL1Flags->bCyaMaskL1;
-    *bEnableAspmDtL1 = pBifAspmL1Flags->bEnableAspmDtL1;
-
-_getAspmL1FlagsSendRpc_exit:
-    portMemFree(pBifAspmL1Flags);
-    return rmStatus;
-}
-
 //
 // HOST RM Device Controls
 //
@@ -177,7 +133,7 @@ deviceCtrlCmdHostGetCapsV2_SOC
     NV0080_CTRL_HOST_GET_CAPS_V2_PARAMS *pHostCapsParamsV2
 )
 {
-    portMemSet(pHostCapsParamsV2, '\0', sizeof(pHostCapsParamsV2));
+    portMemSet(pHostCapsParamsV2, '\0', sizeof(*pHostCapsParamsV2));
 
     return NV_OK;
 }
@@ -211,40 +167,6 @@ subdeviceCtrlCmdBusGetPciInfo_IMPL
     return NV_OK;
 }
 
-NV_STATUS
-subdeviceCtrlCmdBusGetAspmDisableFlags_IMPL
-(
-    Subdevice *pSubdevice,
-    NV2080_CTRL_BUS_GET_ASPM_DISABLE_FLAGS_PARAMS *pParams
-)
-{
-    OBJGPU    *pGpu   = GPU_RES_GET_GPU(pSubdevice);
-    OBJSYS    *pSys   = SYS_GET_INSTANCE();
-    OBJCL     *pCl    = SYS_GET_CL(pSys);
-    NV_STATUS  rmStatus;
-    NvBool     bCyaMaskL1, bEnableAspmDtL1;
-
-    // Send RPC to GSP to obtain BIF PDB values.
-    rmStatus = _getAspmL1FlagsSendRpc(pGpu, &bCyaMaskL1, &bEnableAspmDtL1);
-    if (NV_OK != rmStatus)
-    {
-        return rmStatus;
-    }
-
-    pParams->aspmDisableFlags[NV2080_CTRL_ASPM_DISABLE_FLAGS_L1_MASK_REGKEY_OVERRIDE]                = bCyaMaskL1;
-    // This flag correnpond to an deprecated PDB_PROP_OS_RM_MAKES_POLICY_DECISIONS property which is always returing TURE on non-MACOSX.
-    pParams->aspmDisableFlags[NV2080_CTRL_ASPM_DISABLE_FLAGS_OS_RM_MAKES_POLICY_DECISIONS]           = NV_TRUE;
-    pParams->aspmDisableFlags[NV2080_CTRL_ASPM_DISABLE_FLAGS_GPU_BEHIND_BRIDGE]                      = pGpu->getProperty(pGpu, PDB_PROP_GPU_BEHIND_BRIDGE);
-    pParams->aspmDisableFlags[NV2080_CTRL_ASPM_DISABLE_FLAGS_GPU_UPSTREAM_PORT_L1_UNSUPPORTED]       = pGpu->getProperty(pGpu, PDB_PROP_GPU_UPSTREAM_PORT_L1_UNSUPPORTED);
-    pParams->aspmDisableFlags[NV2080_CTRL_ASPM_DISABLE_FLAGS_GPU_UPSTREAM_PORT_L1_POR_SUPPORTED]     = pGpu->getProperty(pGpu, PDB_PROP_GPU_UPSTREAM_PORT_L1_POR_SUPPORTED);
-    pParams->aspmDisableFlags[NV2080_CTRL_ASPM_DISABLE_FLAGS_GPU_UPSTREAM_PORT_L1_POR_MOBILE_ONLY]   = pGpu->getProperty(pGpu, PDB_PROP_GPU_UPSTREAM_PORT_L1_POR_MOBILE_ONLY);
-    pParams->aspmDisableFlags[NV2080_CTRL_ASPM_DISABLE_FLAGS_CL_ASPM_L1_CHIPSET_DISABLED]            = pCl->getProperty(pCl, PDB_PROP_CL_ASPM_L1_CHIPSET_DISABLED);
-    pParams->aspmDisableFlags[NV2080_CTRL_ASPM_DISABLE_FLAGS_CL_ASPM_L1_CHIPSET_ENABLED_MOBILE_ONLY] = pCl->getProperty(pCl, PDB_PROP_CL_ASPM_L1_CHIPSET_ENABLED_MOBILE_ONLY);
-    pParams->aspmDisableFlags[NV2080_CTRL_ASPM_DISABLE_FLAGS_BIF_ENABLE_ASPM_DT_L1]                  = bEnableAspmDtL1;
-
-    return NV_OK;
-}
-
 NV_STATUS
 subdeviceCtrlCmdBusGetNvlinkPeerIdMask_IMPL
 (
@@ -294,6 +216,8 @@ getBusInfos(OBJGPU *pGpu, NV2080_CTRL_BUS_INFO *pBusInfos, NvU32 busInfoListSize
     {
         NvBool bSendRpc = NV_FALSE;
 
+        NV_CHECK_OK_OR_RETURN(LEVEL_INFO, gpuValidateBusInfoIndex_HAL(pGpu, pBusInfos[i].index));
+
         switch (pBusInfos[i].index)
         {
             case NV2080_CTRL_BUS_INFO_INDEX_PCIE_GEN_INFO:
@@ -747,3 +671,13 @@ subdeviceCtrlCmdBusSetC2CLpwrStateVote_VF
     return NV_ERR_NOT_SUPPORTED;
 }
 
+NV_STATUS
+subdeviceCtrlCmdBusSetC2CIdleThreshold_VF
+(
+    Subdevice                                              *pSubdevice,
+    NV2080_CTRL_CMD_BUS_SET_C2C_LPWR_IDLE_THRESHOLD_PARAMS *pParams
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
