@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -23,9 +23,6 @@
 
 #define NVOC_KERNEL_NVLINK_H_PRIVATE_ACCESS_ALLOWED
 
-// FIXME XXX
-#define NVOC_KERNEL_IOCTRL_H_PRIVATE_ACCESS_ALLOWED
-
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
 #include "kernel/gpu/nvlink/kernel_ioctrl.h"
 #include "kernel/gpu/mem_sys/kern_mem_sys.h"
@@ -82,25 +79,44 @@ _knvlinkFilterDiscoveredLinks
     KernelNvlink *pKernelNvlink
 )
 {
-    // Ensure any vbios disabled links are removed from discovered
-    if (pKernelNvlink->vbiosDisabledLinkMask)
-    {
-        // Update the link mask if overridden through vbios
-        pKernelNvlink->discoveredLinks &= ~(pKernelNvlink->vbiosDisabledLinkMask);
+    NVLINK_BIT_VECTOR localBitVector;
 
-        NV_PRINTF(LEVEL_INFO,
-                  "Links discovered after VBIOS overrides = 0x%x\n",
-                  pKernelNvlink->discoveredLinks);
+    // Ensure any vbios disabled links are removed from discovered
+    if (!bitVectorTestAllCleared(&pKernelNvlink->vbiosDisabledLinkMask))
+    {
+        bitVectorClrAll(&localBitVector);
+
+        //
+        // Invert the vbiosDisabledLinkMask then AN with the discovered links
+        // to get the final mask of links that are discovered
+        //
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, bitVectorComplement(&localBitVector, &pKernelNvlink->vbiosDisabledLinkMask));
+
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, bitVectorAnd(&pKernelNvlink->discoveredLinks,
+                &pKernelNvlink->discoveredLinks, &localBitVector));
+
+        NV_BITVECTOR_PRINT(
+            NV_PRINTF(LEVEL_INFO, "Discovered Links:\n"),
+            &pKernelNvlink->discoveredLinks);
     }
 
     // Filter links that are disabled through regkey overrides
-    if (pKernelNvlink->regkeyDisabledLinksMask)
+    if (!bitVectorTestAllCleared(&pKernelNvlink->regkeyDisabledLinksMask))
     {
-        pKernelNvlink->discoveredLinks &= ~(pKernelNvlink->regkeyDisabledLinksMask);
+        bitVectorClrAll(&localBitVector);
 
-        NV_PRINTF(LEVEL_INFO,
-                  "Links after applying disable links regkey = 0x%x\n",
-                  pKernelNvlink->discoveredLinks);
+        //
+        // Invert the regkeyDisabledLinksMask then AND with the discovered links
+        // to get the final mask of links that are discovered
+        //
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                bitVectorComplement(&localBitVector, &pKernelNvlink->regkeyDisabledLinksMask));
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                bitVectorAnd(&pKernelNvlink->discoveredLinks, &pKernelNvlink->discoveredLinks, &localBitVector));
+
+        NV_BITVECTOR_PRINT(
+                NV_PRINTF(LEVEL_INFO, "Links after applying disable links regkey\n"),
+                &pKernelNvlink->discoveredLinks);
     }
 
     return NV_OK;
@@ -135,13 +151,15 @@ _knvlinkFilterIoctrls
             continue;
         }
 
-        pKernelIoctrl->localDiscoveredLinks &=
-                 KIOCTRL_LINK_GLOBAL_TO_LOCAL_MASK(pKernelNvlink->discoveredLinks);
+        NvU32 localDiscoveredLinks = kioctrlGetLocalDiscoveredLinks(pGpu, pKernelIoctrl);
+
+        localDiscoveredLinks &=
+            kioctrlGetGlobalToLocalMask(pGpu, pKernelIoctrl, KNVLINK_BITVECTOR_TO_MASK(pKernelNvlink, discoveredLinks, 32));
 
         // No need to handle the IOCTRL if no links are being enabled
-        if (pKernelIoctrl->localDiscoveredLinks == 0x0)
+        if (localDiscoveredLinks == 0x0)
         {
-            pKernelNvlink->ioctrlMask &= ~(NVBIT(pKernelIoctrl->PublicId));
+            pKernelNvlink->ioctrlMask &= ~(NVBIT(kioctrlGetPublicId(pGpu, pKernelIoctrl)));
         }
     }
     FOR_EACH_INDEX_IN_MASK_END;
@@ -169,6 +187,9 @@ knvlinkConstructEngine_IMPL
     NV_STATUS status    = NV_OK;
     NvU32     ioctrlIdx = 0;
 
+    // Must clear out all BIT_VECTOR masks to initialize them to 0
+    bitVectorClrAll(&pKernelNvlink->discoveredLinks);
+
     // Initialize the nvlink core library
     knvlinkCoreDriverLoadWar(pGpu, pKernelNvlink);
 
@@ -231,6 +252,16 @@ knvlinkConstructEngine_IMPL
             PDB_PROP_KNVLINK_MINION_GFW_BOOT, NV_FALSE);
     }
 
+    {
+        NVLINK_UNCONTAINED_ERROR_RECOVERY_INFO *pInfo = gpumgrGetNvlinkRecoveryInfo(gpuGetDBDF(pGpu));
+
+        NV_ASSERT_OR_RETURN(pInfo != NULL, NV_ERR_INVALID_STATE);
+        // Mark recovery info as valid for use by error recovery workqueues
+        pInfo->bValid = NV_TRUE;
+        pInfo->DomainBusDevice = gpuGetDBDF(pGpu);
+        pInfo->active = 0;
+    }
+
     return NV_OK;
 }
 
@@ -253,8 +284,8 @@ knvlinkIsPresent_IMPL
 {
     NV_STATUS status = NV_OK;
 
-    // Mark NVLINK as absent when HCC is enabled
-    if (gpuIsCCFeatureEnabled(pGpu))
+    // Mark NVLINK as absent when HCC SPT is enabled
+    if (gpuIsCCFeatureEnabled(pGpu) && (!gpuIsCCMultiGpuProtectedPcieModeEnabled(pGpu) && !gpuIsCCMultiGpuNvleModeEnabled(pGpu)))
         return NV_FALSE;
 
     // On GSP clients, retrieve all device discovery info from GSP through RPC
@@ -347,9 +378,10 @@ knvlinkStateLoad_IMPL
     KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
     NvBool            bMIGNvLinkP2PDisabled = ((pKernelMIGManager != NULL) &&
                                                !kmigmgrIsMIGNvlinkP2PSupported(pGpu, pKernelMIGManager));
-    NvU32             preInitializedLinks;
+    NvU64             preInitializedLinks;
     NvU32             i;
     OBJTMR            *pTmr = GPU_GET_TIMER(pGpu);
+    NVLINK_BIT_VECTOR localBitVector;
 
     //
     // If we are on the resume path, nvlinkIsPresent will not be called,
@@ -432,10 +464,15 @@ knvlinkStateLoad_IMPL
     }
 
     // Remove the init disabled links from the discovered links mask
-    pKernelNvlink->discoveredLinks &= ~(pKernelNvlink->initDisabledLinksMask);
+    bitVectorClrAll(&localBitVector);
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+        convertMaskToBitVector(~((NvU64)pKernelNvlink->initDisabledLinksMask), &localBitVector));
+
+    bitVectorAnd(&pKernelNvlink->discoveredLinks, &pKernelNvlink->discoveredLinks, &localBitVector);
 
     // Track un-connected links, we assume all discovered links are connected.
-    pKernelNvlink->connectedLinksMask = pKernelNvlink->discoveredLinks;
+    bitVectorCopy(&pKernelNvlink->connectedLinksMask, &pKernelNvlink->discoveredLinks);
 
     // Initialize initializedLinks to 0 (assume no links initialized)
     pKernelNvlink->initializedLinks = 0;
@@ -466,7 +503,7 @@ knvlinkStateLoad_IMPL
     // At this point, the discovered links mask is filtered. If there are no
     // discovered links, then we skip the rest of the steps.
     //
-    if (pKernelNvlink->discoveredLinks == 0)
+    if (bitVectorTestAllCleared(&pKernelNvlink->discoveredLinks))
     {
         goto knvlinkStateLoad_end;
     }
@@ -489,31 +526,33 @@ knvlinkStateLoad_IMPL
     //
     if (pKernelNvlink->bRegistryLinkOverride)
     {
-        pKernelNvlink->enabledLinks = pKernelNvlink->discoveredLinks &
-                                      pKernelNvlink->registryLinkMask;
+        NVLINK_BIT_VECTOR registryLinkMaskVec;
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+            convertMaskToBitVector(KNVLINK_GET_MASK(pKernelNvlink, registryLinkMask, 64), &registryLinkMaskVec));
+        bitVectorAnd(&pKernelNvlink->enabledLinks, &pKernelNvlink->discoveredLinks, &registryLinkMaskVec);
     }
     else if (bMIGNvLinkP2PDisabled)
     {
         // NvLink is not supported with MIG
-        pKernelNvlink->enabledLinks = 0;
+        bitVectorClrAll(&pKernelNvlink->enabledLinks);
     }
     else
     {
-        pKernelNvlink->enabledLinks = pKernelNvlink->discoveredLinks;
+        bitVectorCopy(&pKernelNvlink->enabledLinks, &pKernelNvlink->discoveredLinks);
     }
 
     // Sense NVLink bridge presence and remove links on missing bridges.
     knvlinkFilterBridgeLinks_HAL(pGpu, pKernelNvlink);
 
     // Disconnected links mask tracks links whose remote ends are not discovered
-    pKernelNvlink->disconnectedLinkMask = pKernelNvlink->enabledLinks;
+    pKernelNvlink->disconnectedLinkMask = KNVLINK_BITVECTOR_TO_MASK(pKernelNvlink, enabledLinks, 64);
 
     if (!IS_RTLSIM(pGpu) || pKernelNvlink->bForceEnableCoreLibRtlsims)
     {
         if (!knvlinkPoweredUpForD3_HAL(pGpu, pKernelNvlink))
         {
             // Register links in the nvlink core library
-            FOR_EACH_INDEX_IN_MASK(32, i, pKernelNvlink->enabledLinks)
+            FOR_EACH_IN_BITVECTOR(&pKernelNvlink->enabledLinks, i)
             {
                 status = knvlinkCoreAddLink(pGpu, pKernelNvlink, i);
                 if (status != NV_OK)
@@ -523,7 +562,7 @@ knvlinkStateLoad_IMPL
                     goto knvlinkStateLoad_end;
                 }
             }
-            FOR_EACH_INDEX_IN_MASK_END;
+            FOR_EACH_IN_BITVECTOR_END();
         }
         else
         {
@@ -621,12 +660,13 @@ knvlinkStateLoad_IMPL
     }
 
     if (!knvlinkIsForcedConfig(pGpu, pKernelNvlink) && pKernelNvlink->bEnableAli &&
+        (pKernelNvlink->ipVerNvlink < NVLINK_VERSION_50) &&
         (!pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_MINION_GFW_BOOT) ||
           pKernelNvlink->getProperty(pKernelNvlink,
                                      PDB_PROP_KNVLINK_MINION_FORCE_ALI_TRAINING)))
     {
         status = knvlinkPreTrainLinksToActiveAli(pGpu, pKernelNvlink,
-                                                 pKernelNvlink->enabledLinks, NV_TRUE);
+                                                 KNVLINK_BITVECTOR_TO_MASK(pKernelNvlink, enabledLinks, 32), NV_TRUE);
         if (status != NV_OK)
         {
             goto knvlinkStateLoad_end;
@@ -638,7 +678,7 @@ knvlinkStateLoad_IMPL
         // will be queries via DLSTAT to know their status and training
         // progression.
         //
-        FOR_EACH_INDEX_IN_MASK(32, i, pKernelNvlink->enabledLinks)
+        FOR_EACH_IN_BITVECTOR(&pKernelNvlink->enabledLinks, i)
         {
             status = knvlinkTrainLinksToActiveAli(pGpu, pKernelNvlink, NVBIT(i), NV_FALSE);
             if (status != NV_OK)
@@ -659,10 +699,10 @@ knvlinkStateLoad_IMPL
                 osDelayUs(8000000);
             }
         }
-        FOR_EACH_INDEX_IN_MASK_END;
+        FOR_EACH_IN_BITVECTOR_END();
     }
 
-    FOR_EACH_INDEX_IN_MASK(32, i, pKernelNvlink->enabledLinks)
+    FOR_EACH_IN_BITVECTOR(&pKernelNvlink->enabledLinks, i)
     {
         status = tmrEventCreate(pTmr, &pKernelNvlink->nvlinkLinks[i].pTmrEvent,
                             ioctrlFaultUpTmrHandler, NULL,
@@ -673,10 +713,17 @@ knvlinkStateLoad_IMPL
                           "Failed to create TmrEvent for Link %d\n", i);
         }
     }
-    FOR_EACH_INDEX_IN_MASK_END;
+    FOR_EACH_IN_BITVECTOR_END();
 
     listInit(&pKernelNvlink->faultUpLinks, portMemAllocatorGetGlobalNonPaged());
 
+    knvlinkDumpCallbackRegister_HAL(pGpu, pKernelNvlink);
+
+    knvlinkPostSchedulingEnableCallbackRegister_HAL(pGpu, pKernelNvlink);
+
+    // Update list of HSHUB supported RBM modes
+    knvlinkGetHshubSupportedRbmModes_HAL(pGpu, pKernelNvlink);
+
 knvlinkStateLoad_end:
 
     if (status != NV_OK)
@@ -770,12 +817,34 @@ knvlinkStatePostLoad_IMPL
                 gpuInstance = 0;
                 while ((pRemoteGpu = gpumgrGetNextGpu(gpuMask, &gpuInstance)) != NULL)
                 {
-                    knvlinkTrainP2pLinksToActive(pGpu, pRemoteGpu, pKernelNvlink);
+                    if (gpuIsStateLoaded(pRemoteGpu) || gpuIsStateLoading(pRemoteGpu))
+                    {
+                        knvlinkTrainP2pLinksToActive(pGpu, pRemoteGpu, pKernelNvlink);
+                    }
                 }
             }
         }
     }
 
+    // We check for NVLE enablement here for the production/non-MODS case
+    if (pKernelNvlink->bNvleModeRegkey)
+    {
+        //
+        // On new firmware, CC enabled implies NVLE enabled, which is not true on older
+        // firmware. Hence confirm if we need to check for NVLE enablement
+        //
+        NV_ASSERT_OK_OR_RETURN(knvlinkGetEncryptionBits_HAL(pGpu, pKernelNvlink));
+
+        if (RMCFG_FEATURE_MODS_FEATURES || pKernelNvlink->bMmuNvlinkEncryptEn)
+        {
+            if (!knvlinkIsNvleEnabled_HAL(pGpu, pKernelNvlink))
+            {
+                NV_PRINTF(LEVEL_ERROR," NVLE not enabled on GPU%d\n", pGpu->gpuInstance);
+                return NV_ERR_FEATURE_NOT_ENABLED;
+            }
+        }
+    }
+
     status = knvlinkStatePostLoadHal_HAL(pGpu, pKernelNvlink);
     if (status != NV_OK)
     {
@@ -783,6 +852,11 @@ knvlinkStatePostLoad_IMPL
         return status;
     }
 
+    if (!GPU_IS_NVSWITCH_DETECTED(pGpu))
+    {
+        NV_ASSERT_OK_OR_RETURN(knvlinkSetDirectConnectBaseAddress_HAL(pGpu, pKernelNvlink));
+    }
+
     return NV_OK;
 }
 
@@ -812,6 +886,8 @@ knvlinkStateUnload_IMPL
         kbusDestroyFla_HAL(pGpu, GPU_GET_KERNEL_BUS(pGpu));
     }
 
+    knvlinkPostSchedulingEnableCallbackUnregister_HAL(pGpu, pKernelNvlink);
+
     return NV_OK;
 }
 
@@ -833,6 +909,7 @@ knvlinkStatePostUnload_IMPL
 {
     OBJSYS    *pSys   = SYS_GET_INSTANCE();
     NV_STATUS  status = NV_OK;
+
 #if defined(INCLUDE_NVLINK_LIB)
     NvU32 linkId = 0;
 #endif
@@ -904,14 +981,14 @@ knvlinkStatePostUnload_IMPL
                         !pSys->getProperty(pSys, PDB_PROP_SYS_FABRIC_IS_EXTERNALLY_MANAGED) &&
                         !API_GPU_IN_RESET_SANITY_CHECK(pGpu))
     {
-        NV2080_CTRL_NVLINK_DISABLE_DL_INTERRUPTS_PARAMS params;
+        NV2080_CTRL_INTERNAL_NVLINK_DISABLE_DL_INTERRUPTS_PARAMS params;
 
         portMemSet(&params, 0, sizeof(params));
-        params.linkMask = pKernelNvlink->enabledLinks;
+        params.linkMask = KNVLINK_BITVECTOR_TO_MASK(pKernelNvlink, enabledLinks, 32);
 
         // Disable all the DL interrupts
         status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                     NV2080_CTRL_CMD_NVLINK_DISABLE_DL_INTERRUPTS,
+                                     NV2080_CTRL_CMD_INTERNAL_NVLINK_DISABLE_DL_INTERRUPTS,
                                      (void *)&params, sizeof(params));
         if (status != NV_OK)
         {
@@ -931,7 +1008,7 @@ knvlinkStatePostUnload_IMPL
     }
 
 #if defined(INCLUDE_NVLINK_LIB)
-    FOR_EACH_INDEX_IN_MASK(32, linkId, pKernelNvlink->enabledLinks)
+    FOR_EACH_IN_BITVECTOR(&pKernelNvlink->enabledLinks, linkId)
     {
         // Update remote GPU disconnectedLinkMasks
         OBJGPU *pRemoteGpu = gpumgrGetGpuFromBusInfo(pKernelNvlink->nvlinkLinks[linkId].remoteEndInfo.domain,
@@ -941,14 +1018,21 @@ knvlinkStatePostUnload_IMPL
         {
             KernelNvlink *pRemoteKernelNvlink = GPU_GET_KERNEL_NVLINK(pRemoteGpu);
 
-            pRemoteKernelNvlink->disconnectedLinkMask |= NVBIT(pKernelNvlink->nvlinkLinks[linkId].remoteEndInfo.linkNumber);
+            pRemoteKernelNvlink->disconnectedLinkMask |= NVBIT64(pKernelNvlink->nvlinkLinks[linkId].remoteEndInfo.linkNumber);
         }
     }
-    FOR_EACH_INDEX_IN_MASK_END;
+    FOR_EACH_IN_BITVECTOR_END();
 #endif
 
     listDestroy(&pKernelNvlink->faultUpLinks);
 
+    // If Nvlink encryption is enabled, clear the NVLE boolean state
+    status = knvlinkClearEncryptionKeys(pGpu, pKernelNvlink);
+    if ((status != NV_OK) && (status != NV_ERR_NOT_SUPPORTED))
+    {
+        NV_PRINTF(LEVEL_ERROR, "Failed to clear NVLE keys for the GPU\n");
+    }
+
 knvlinkStatePostUnload_end:
 
     _knvlinkPurgeState(pGpu, pKernelNvlink);
@@ -981,19 +1065,15 @@ _knvlinkPurgeState
     NvBool bMIGNvLinkP2PDisabled = ((pKernelMIGManager != NULL) &&
                           !kmigmgrIsMIGNvlinkP2PSupported(pGpu, pKernelMIGManager));
 
-    FOR_EACH_INDEX_IN_MASK(32, linkId, pKernelNvlink->enabledLinks)
+    FOR_EACH_IN_BITVECTOR(&pKernelNvlink->discoveredLinks, linkId)
     {
         if ((pKernelNvlink->nvlinkLinks[linkId].pTmrEvent != NULL) && (pTmr != NULL))
         {
-            if (tmrEventOnList(pTmr, pKernelNvlink->nvlinkLinks[linkId].pTmrEvent))
-            {
-                 tmrEventCancel(pTmr, pKernelNvlink->nvlinkLinks[linkId].pTmrEvent);
-            }
             tmrEventDestroy(pTmr, pKernelNvlink->nvlinkLinks[linkId].pTmrEvent);
             pKernelNvlink->nvlinkLinks[linkId].pTmrEvent = NULL;
         }
     }
-    FOR_EACH_INDEX_IN_MASK_END;
+    FOR_EACH_IN_BITVECTOR_END();
 
     // RM disables NVLink at runtime in Hopper so device un-registration can't be skipped
     if (!IsGH100orBetter(pGpu))
@@ -1019,17 +1099,29 @@ _knvlinkPurgeState
         if (pKernelNvlink->pNvlinkDev)
         {
             // Un-register the links from nvlink core library
-            FOR_EACH_INDEX_IN_MASK(32, linkId, pKernelNvlink->enabledLinks)
+            FOR_EACH_IN_BITVECTOR(&pKernelNvlink->enabledLinks, linkId)
             {
                 knvlinkCoreRemoveLink(pGpu, pKernelNvlink, linkId);
             }
-            FOR_EACH_INDEX_IN_MASK_END;
+            FOR_EACH_IN_BITVECTOR_END();
 
             // Un-register the nvgpu device from nvlink core library
             knvlinkCoreRemoveDevice(pGpu, pKernelNvlink);
         }
     }
 
+    //
+    // pGidString is allocated within knvlinkStatePostLoad -> knvlinkCoreUpdateDeviceUUID
+    // so need to free it during destruct
+    // Freeing it within knvlinkCoreRemoveDevice could create problems if
+    // AddDevice/RemoveDevice are used outside StateLoad/StatePostUnload/StateDestroy in the future
+    //
+    if (pKernelNvlink->pGidString)
+    {
+        portMemFree(pKernelNvlink->pGidString);
+        pKernelNvlink->pGidString = NULL;
+    }
+
 _knvlinkPurgeState_end:
 
 #endif
@@ -1059,6 +1151,33 @@ _knvlinkPurgeState_end:
     return NV_OK;
 }
 
+/*!
+ * @brief Workitem to shutdown links async to any error
+ * paths that were taken
+ *
+ * @param[in] pGpu           OBJGPU pointer
+ * @param[in] pKernelNvlink  KernelNvlink pointer
+ *
+ */
+void
+knvlinkShutdownLinks_WORKITEM
+(
+    NvU32 gpuInstance,
+    void *pData
+)
+{
+    OBJGPU *pGpu                = gpumgrGetGpu(gpuInstance);
+    KernelNvlink *pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
+
+    // Sanity Checks
+    NV_CHECK_OR_RETURN_VOID(LEVEL_INFO,
+            (pGpu != NULL &&
+             pKernelNvlink != NULL));
+
+    NV_ASSERT_OR_RETURN_VOID(
+        knvlinkCoreShutdownDeviceLinks(pGpu, pKernelNvlink, NV_TRUE) == NV_OK);
+}
+
 /*!
  * @brief Degraded Mode will be set if other end of the linkId
  *        is not degraded.
@@ -1074,15 +1193,15 @@ knvlinkSetDegradedMode_IMPL
 (
     OBJGPU       *pGpu,
     KernelNvlink *pKernelNvlink,
-    NvU32         linkId
+    NvS32         linkId
 )
 {
-    NvU32         status = NV_ERR_GENERIC;
-    NvU32         gpuInstance;
-    OBJGPU       *pRemoteGpu = NULL;
-    KernelNvlink *pRemoteKernelNvlink = NULL;
+    NV_STATUS status = NV_OK;
 
-    if (!pKernelNvlink)
+    // Prevent invalid shift later
+    NV_ASSERT_OR_RETURN_VOID((linkId == -1) || ((linkId >= 0) && (linkId < 32)));
+
+    if (pKernelNvlink == NULL)
     {
         NV_PRINTF(LEVEL_ERROR,
                 "Failed to get Local Nvlink info for linkId %d to update Degraded GPU%d status\n",
@@ -1091,57 +1210,59 @@ knvlinkSetDegradedMode_IMPL
         return;
     }
 
-    if(pKernelNvlink->bIsGpuDegraded)
+    if (pKernelNvlink->bIsGpuDegraded)
     {
         return;
     }
 
-    //Find the remote GPU/NVLink attached to this link, if any
-    for (gpuInstance = 0; gpuInstance < NV_MAX_DEVICES; gpuInstance++)
-    {
-        if (pKernelNvlink->peerLinkMasks[gpuInstance] & NVBIT(linkId))
-        {
-            pRemoteGpu = gpumgrGetGpu(gpuInstance);
-            break;
-        }
-    }
+    pKernelNvlink->bIsGpuDegraded = NV_TRUE;
+    NV_PRINTF(LEVEL_ERROR,
+            "GPU%d marked Degraded. Error originated on linkId %d!\n",
+            pGpu->gpuInstance, linkId);
 
-    if (pRemoteGpu == NULL)
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                "Failed to get Remote GPU info for linkId %d to update Degraded GPU%d status\n",
-                linkId, pGpu->gpuInstance);
+    // Queue a workitem to handle rest of the degraded mode handling
 
-        return;
-    }
+    NV_CHECK_OK_OR_ELSE(status,
+        LEVEL_ERROR,
+        osQueueWorkItem(pGpu,
+                        knvlinkShutdownLinks_WORKITEM,
+                        NULL,
+                        (OsQueueWorkItemFlags){
+                            .apiLock = WORKITEM_FLAGS_API_LOCK_READ_ONLY,
+                            .bLockGpuGroupSubdevice = NV_TRUE}),
+        return;);
+}
 
-    pRemoteKernelNvlink = GPU_GET_KERNEL_NVLINK(pRemoteGpu);
-    if (!pRemoteKernelNvlink)
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                "Failed to get Remote Nvlink info for linkId %d to update Degraded GPU%d status\n",
-                linkId, pGpu->gpuInstance);
+/*!
+ * @brief Gets degraded mode for current GPU
+ *
+ * @param[in] pGpu           OBJGPU pointer
+ * @param[in] pKernelNvlink  KernelNvlink pointer
+ *
+ * @return Current NVLink degraded mode
+ */
+NvBool
+knvlinkGetDegradedMode_IMPL
+(
+    OBJGPU       *pGpu,
+    KernelNvlink *pKernelNvlink
+)
+{
+    return pKernelNvlink->bIsGpuDegraded;
+}
 
-        return;
-    }
-
-    if (pRemoteKernelNvlink->bIsGpuDegraded == NV_FALSE)
-    {
-        pKernelNvlink->bIsGpuDegraded = NV_TRUE;
-        NV_PRINTF(LEVEL_ERROR,
-                "GPU%d marked Degraded for error on linkId %d \n",
-                pGpu->gpuInstance, linkId);
-
-        // shutdown all the links on this GPU
-        status = knvlinkCoreShutdownDeviceLinks(pGpu, pKernelNvlink, NV_TRUE);
-        if (status != NV_OK)
-        {
-           NV_PRINTF(LEVEL_ERROR,
-                     "failed to shutdown links on degraded GPU%d\n", pGpu->gpuInstance);
-        }
-    }
-
-    return;
+/*!
+ * @brief Is NVLINK Uncontained error recovery active?
+ */
+NvBool
+knvlinkIsUncontainedErrorRecoveryActive_IMPL
+(
+    OBJGPU       *pGpu,
+    KernelNvlink *pKernelNvlink
+)
+{
+    NVLINK_UNCONTAINED_ERROR_RECOVERY_INFO *pInfo = gpumgrGetNvlinkRecoveryInfo(gpuGetDBDF(pGpu));
+    return ((pInfo != NULL) && pInfo->bValid && (portAtomicOrU32(&pInfo->active, 0) != 0));
 }
 
 void
@@ -1192,26 +1313,26 @@ _knvlinkProcessSysmemLinks
 {
     NV_STATUS status = NV_OK;
 
-#if defined(NVCPU_PPC64LE) || defined(NVCPU_AARCH64)
+#if defined(NVCPU_AARCH64)
     if (pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_SYSMEM_SUPPORT_ENABLED))
     {
         //
-        // In case of IBM or Tegra, the sysmem links will already have
+        // In case of Tegra, the sysmem links will already have
         // been registered in nvlink core library. In order to trigger
         // topology detection, call knvlinkCoreGetRemoteDeviceInfo
         //
         if (!knvlinkIsForcedConfig(pGpu, pKernelNvlink) && !pKernelNvlink->pLinkConnection)
         {
             //
-            // Establish the current link topology and enable IBM CPU/SYSMEM links.
-            // If any of the discovered links are CPU/SYSMEM, they will be trained,
+            // Establish the current link topology and enable SYSMEM links.
+            // If any of the discovered links are SYSMEM, they will be trained,
             // post-enabled, and then enabled in HSHUB when the call has completed.
             //
             status = knvlinkCoreGetRemoteDeviceInfo(pGpu, pKernelNvlink);
             if (status != NV_OK)
             {
                 NV_PRINTF(LEVEL_ERROR,
-                          "Failed call to get remote device info during IBM CPU/SYSMEM links "
+                          "Failed call to get remote device info during SYSMEM links "
                           "setup, failing NVLink StateLoad on GPU%d!!!\n\n",
                           pGpu->gpuInstance);
 
@@ -1253,15 +1374,15 @@ _knvlinkProcessSysmemLinks
     }
 
     // Set Buffer ready for the sysmem links
-    NV2080_CTRL_NVLINK_PROGRAM_BUFFERREADY_PARAMS programBufferRdyParams;
+    NV2080_CTRL_INTERNAL_NVLINK_PROGRAM_BUFFERREADY_PARAMS programBufferRdyParams;
 
     portMemSet(&programBufferRdyParams, 0, sizeof(programBufferRdyParams));
-    programBufferRdyParams.flags        = NV2080_CTRL_NVLINK_PROGRAM_BUFFERREADY_FLAGS_SET;
+    programBufferRdyParams.flags        = NV2080_CTRL_INTERNAL_NVLINK_PROGRAM_BUFFERREADY_FLAGS_SET;
     programBufferRdyParams.bSysmem      = NV_TRUE;
     programBufferRdyParams.peerLinkMask = 0;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_PROGRAM_BUFFERREADY,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_PROGRAM_BUFFERREADY,
                                  (void *)&programBufferRdyParams,
                                  sizeof(programBufferRdyParams));
     if (status != NV_OK)
@@ -1286,11 +1407,11 @@ _knvlinkProcessSysmemLinks
         // PPC+GV100.
         //
 
-        NV2080_CTRL_NVLINK_ENABLE_SYSMEM_NVLINK_ATS_PARAMS sysmemNvlinkAtsParams;
+        NV2080_CTRL_INTERNAL_NVLINK_ENABLE_SYSMEM_NVLINK_ATS_PARAMS sysmemNvlinkAtsParams;
         portMemSet(&sysmemNvlinkAtsParams, 0, sizeof(sysmemNvlinkAtsParams));
 
         status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                     NV2080_CTRL_CMD_NVLINK_ENABLE_SYSMEM_NVLINK_ATS,
+                                     NV2080_CTRL_CMD_INTERNAL_NVLINK_ENABLE_SYSMEM_NVLINK_ATS,
                                      (void *)&sysmemNvlinkAtsParams,
                                      sizeof(sysmemNvlinkAtsParams));
         if (status != NV_OK)
@@ -1301,8 +1422,6 @@ _knvlinkProcessSysmemLinks
 
         status = kmemsysSetupCoherentCpuLink(pGpu, GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu), bFlush);
         NV_ASSERT_OR_RETURN(status == NV_OK, status);
-
-        osSetNVLinkSysmemLinkState(pGpu, NV_TRUE);
     }
 
     return status;
